/*
 * LinShare is an open source filesharing software, part of the LinPKI software
 * suite, developed by Linagora.
 * 
 * Copyright (C) 2014 LINAGORA
 * 
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version, provided you comply with the Additional Terms applicable for
 * LinShare software by Linagora pursuant to Section 7 of the GNU Affero General
 * Public License, subsections (b), (c), and (e), pursuant to which you must
 * notably (i) retain the display of the “LinShare™” trademark/logo at the top
 * of the interface window, the display of the “You are using the Open Source
 * and free version of LinShare™, powered by Linagora © 2009–2014. Contribute to
 * Linshare R&D by subscribing to an Enterprise offer!” infobox and in the
 * e-mails sent with the Program, (ii) retain all hypertext links between
 * LinShare and linshare.org, between linagora.com and Linagora, and (iii)
 * refrain from infringing Linagora intellectual property rights over its
 * trademarks and commercial brands. Other Additional Terms apply, see
 * <http://www.linagora.com/licenses/> for more details.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License and
 * its applicable Additional Terms for LinShare along with this program. If not,
 * see <http://www.gnu.org/licenses/> for the GNU Affero General Public License
 * version 3 and <http://www.linagora.com/licenses/> for the Additional Terms
 * applicable to LinShare software.
 */
package org.linagora.linshare.core.business.service.impl;

import java.util.Set;

import org.linagora.linshare.core.business.service.AnonymousUrlBusinessService;
import org.linagora.linshare.core.domain.entities.AnonymousShareEntry;
import org.linagora.linshare.core.domain.entities.AnonymousUrl;
import org.linagora.linshare.core.domain.entities.Contact;
import org.linagora.linshare.core.exception.BusinessException;
import org.linagora.linshare.core.exception.LinShareNotSuchElementException;
import org.linagora.linshare.core.repository.AnonymousUrlRepository;
import org.linagora.linshare.core.service.PasswordService;
import org.linagora.linshare.core.utils.HashUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class AnonymousUrlBusinessServiceImpl implements AnonymousUrlBusinessService {

	private final AnonymousUrlRepository anonymousUrlRepository;
	private final String baseSecuredUrl;
	private final PasswordService passwordService;
	
	private static final Logger logger = LoggerFactory.getLogger(AnonymousUrlBusinessServiceImpl.class);
	
	
	public AnonymousUrlBusinessServiceImpl(AnonymousUrlRepository anonymousUrlRepository, String baseSecuredUrl, PasswordService passwordService) {
		super();
		this.anonymousUrlRepository = anonymousUrlRepository;
		this.baseSecuredUrl = baseSecuredUrl;
		this.passwordService = passwordService;
	}


	@Override
	public AnonymousUrl findByUuid(String uuid) {
		return anonymousUrlRepository.findByUuid(uuid);
	}

	
	@Override
	public AnonymousUrl create(Boolean passwordProtected, Contact contact) throws BusinessException {
		
		AnonymousUrl anonymousUrl = new AnonymousUrl(baseSecuredUrl, contact);
		if(passwordProtected) {
			String password = passwordService.generatePassword();
			// We store it temporary in this object for mail notification.
			anonymousUrl.setTemporaryPlainTextPassword(password);
			anonymousUrl.setPassword(HashUtils.hashSha1withBase64(password.getBytes()));
		}
		return anonymousUrlRepository.create(anonymousUrl);
	}


	@Override
	public void update(AnonymousUrl anonymousUrl) throws BusinessException {
		anonymousUrlRepository.update(anonymousUrl);
	}


	@Override
	public AnonymousUrl getAnonymousUrl(String uuid) throws LinShareNotSuchElementException {
		AnonymousUrl anonymousUrl = findByUuid(uuid);
		if(anonymousUrl == null) {
			 throw new LinShareNotSuchElementException("anonymousUrl url not found");
		}
		return anonymousUrl;
	}


	@Override
	public boolean isValidPassword(AnonymousUrl anonymousUrl, String password) {
		if (anonymousUrl == null) throw new IllegalArgumentException("anonymousUrl url cannot be null");

		// Check password validity
		if (password != null) {
			String hashedPassword = HashUtils.hashSha1withBase64(password.getBytes());
			return hashedPassword.equals(anonymousUrl.getPassword());
		}
		return true;
	}


	@Override
	public boolean isExpired(AnonymousUrl anonymousUrl) {
		if (anonymousUrl == null)
			throw new IllegalArgumentException("anonymousUrl url cannot be null");

		Set<AnonymousShareEntry> entries = anonymousUrl.getAnonymousShareEntries();
		
		if(entries != null && entries.size() > 0) {
			return false;
		}
		return true;
	}

	
}
